///////////////////////////////////////////////////////////////////////////////
//
//	app.cpp
//
///////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>

#include	<stdio.h>
#include	<string.h>
#include	<time.h>
#include	"std.h"
#include	"rio.h"

// version 1.01
#define		ID_VERSION				101

// default io base
#define		PORT_BASE_DEFAULT		0x378

// output
#define		INFOSTR					printf
#define		ERRORSTR				INFOSTR

///////////////////////////////////////////////////////////////////////////////
// return pointer to static string containing datetime
static char* TimeStr( long lValue )
{
	static char szBuf[ 64 ];
	struct tm* psDateTime;
	psDateTime = localtime( &lValue );

	if ( !psDateTime )
		strcpy( szBuf, "INVALID DATE/TIME" );
	else
	{
		sprintf(
			szBuf,
			"%02u/%02u/%02u %02u:%02u:%02u",
			(UINT)psDateTime->tm_mday % 100,
			(UINT)psDateTime->tm_mon+1 % 100,
			(UINT)psDateTime->tm_year % 100,
			(UINT)psDateTime->tm_hour % 100,
			(UINT)psDateTime->tm_min % 100,
			(UINT)psDateTime->tm_sec % 100
		);
	}

	return szBuf;
}

///////////////////////////////////////////////////////////////////////////////
// display directory
static void DisplayDirectory( CRio& cRio, BOOL bVerbose )
{
	CDirBlock& cDirBlock = cRio.GetDirectoryBlock();

	CDirHeader& cDirHeader = cDirBlock.m_cDirHeader;
	INFOSTR( "\n" );
	INFOSTR( "  entry count %hu\n", cDirHeader.m_usCountEntry );
	INFOSTR( " total memory %ld KB\n", ((long)cDirHeader.m_usCount32KBlockAvailable * CRIO_SIZE_32KBLOCK) / 1024 );
	INFOSTR( "  used memory %ld KB\n", ((long)cDirHeader.m_usCount32KBlockUsed * CRIO_SIZE_32KBLOCK) / 1024 );
	INFOSTR( "unused memory %ld KB\n", ((long)cDirHeader.m_usCount32KBlockRemaining * CRIO_SIZE_32KBLOCK) / 1024 );
	if ( bVerbose )
	{
		INFOSTR( "  last update %s\n", TimeStr(cDirHeader.m_lTimeLastUpdate) );
		INFOSTR( "    checksum1 0x%04hx\n", cDirHeader.m_usChecksum1 );
		INFOSTR( "    checksum2 0x%04hx\n", cDirHeader.m_usChecksum2 );
	}

	UINT uiCountEntry = cDirHeader.m_usCountEntry;
	if ( uiCountEntry )
	{
		CDirEntry* pDirEntry = cDirBlock.m_acDirEntry;
		if ( uiCountEntry > CRIO_MAX_DIRENTRY )
			uiCountEntry = CRIO_MAX_DIRENTRY;

		// extended output
		if ( bVerbose )
		{
			printf( "\n" );
			printf( "No 32KPos 32KCount Mod32K Size     Upload Date/Time  Title\n" );
			printf( "-----------------------------------------------------------------------------\n" );

			for( UINT uiA=0; uiA<uiCountEntry; ++uiA, ++pDirEntry )
			{
				INFOSTR( "%02u 0x%04hx %-4hu     0x%04hx %-8ld %s %-24.24s\n",
					uiA+1,
					pDirEntry->m_usPos32KBlock,
					pDirEntry->m_usCount32KBlock,
					pDirEntry->m_usSize32KMod,
					pDirEntry->m_lSize,
					TimeStr(pDirEntry->m_lTimeUpload),
					pDirEntry->m_szName
				);
			}
		}
		// normal output
		else
		{
			printf( "\n" );
			printf( "No  Size      Upload Date/Time   Title\n" );
			printf( "-----------------------------------------------------------------------------\n" );

			for( UINT uiA=0; uiA<uiCountEntry; ++uiA, ++pDirEntry )
			{
				INFOSTR( "%02u  %-8ld  %s  %-40.40s\n",
					uiA+1,
					pDirEntry->m_lSize,
					TimeStr(pDirEntry->m_lTimeUpload),
					pDirEntry->m_szName
				);
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// process playlist file
static BOOL ProcessPlaylist( CRio& cRio, char* pszFile, BOOL bVerbose )
{
	// open playlist for read
	FILE* fpFile = fopen( pszFile, "r" );
	if ( !fpFile )
	{
		ERRORSTR( "unable to open '%s' for read\n", pszFile );
		return FALSE;
	}

	// default to complete ok
	BOOL bResult = TRUE;

	// process entries
	char szBuf[ 128 ];
	while( fscanf(fpFile, "%s", szBuf) == 1 )
	{
		if ( bVerbose )
			INFOSTR( "uploading %s\n", szBuf );
		if ( !cRio.TxFile(szBuf, bVerbose) )
		{
			bResult = FALSE;
			break;
		}
	}

	// close playlist file
	fclose( fpFile );

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////
// display help
static void Help( void )
{
	INFOSTR( "Rio Geo version 1.00 \n" );
	INFOSTR( "\n" );
	INFOSTR( "command line switches available :-\n" );
	INFOSTR( "\n" );
	INFOSTR( "	-d display directory\n" );
	INFOSTR( "	-i initialize\n" );
	INFOSTR( "	-u specify file to upload\n" );
	INFOSTR( "	-o specify track to download\n" );
	INFOSTR( "	-f specify text based playlist file which contains files to be upload\n" );
	INFOSTR( "	-p specify parallel port base IO address, default=0x378\n" );
	INFOSTR( "	-v enable verbose mode\n" );
	INFOSTR( "\n" );
	INFOSTR( "examples...\n" );
	INFOSTR( "\n" );
	INFOSTR( "	; display directory using parallel port at 0x278\n" );
	INFOSTR( "	riogeo -p 0x278 -d\n" );
	INFOSTR( "	; initialize and upload file\n" );
	INFOSTR( "	riogeo -i -u song.mp3\n" );
	INFOSTR( "	; initialize and upload files in playlist\n" );
	INFOSTR( "	riogeo -i -f playlist.txt\n" );
	INFOSTR( "	; download file - track 1\n" );
	INFOSTR( "	riogeo -o 1\n" );
}

///////////////////////////////////////////////////////////////////////////////
int main( int iCountArg, char* paszArg[] )
{
	// default settings
	BOOL bDisplayDir = FALSE;
	BOOL bInit = FALSE;
	BOOL bVerbose = FALSE;
	char* pszFileUpload = NULL;
	char* pszFilePlaylist = NULL;
	char* pszTrackNot = NULL;
	int pszTrackNo = 0;
	int iPortBase = PORT_BASE_DEFAULT;

	// process command line args
	if ( iCountArg < 2 )
	{
		Help();
		return FALSE;
	}
	for( int iA=1; iA<iCountArg; ++iA )
	{
		// check for display directory request
		if ( !strcmp(paszArg[iA], "-d") )
			bDisplayDir = TRUE;
		// check for init request
		else if ( !strcmp(paszArg[iA], "-i") )
			bInit = TRUE;
		// check for verbose mode request
		else if ( !strcmp(paszArg[iA], "-v") )
			bVerbose = TRUE;
		// check for upload file request
		else if ( !strcmp(paszArg[iA], "-u") )
		{
			if ( (iA+1) < iCountArg )
			{
				++iA;
				pszFileUpload = paszArg[iA];
			}
		}
		// check for download file request
		else if ( !strcmp(paszArg[iA], "-o") )
		{
			if ( (iA+1) < iCountArg )
			{
				++iA;
				pszTrackNo = atoi( paszArg[iA] );
				pszTrackNot = "here";
			}
		}

		// check for playlist file request
		else if ( !strcmp(paszArg[iA], "-f") )
		{
			if ( (iA+1) < iCountArg )
			{
				++iA;
				pszFilePlaylist = paszArg[iA];
			}
		}
		// check for port base request
		else if ( !strcmp(paszArg[iA], "-p") )
		{
			if ( (iA+1) < iCountArg )
			{
				++iA;
				sscanf( paszArg[iA], "%x", &iPortBase );
			}
		}
		// else help
		else
		{
			Help();
			return FALSE;
		}
	}

	// setup
	CRio cRio;
	if ( !cRio.Set(iPortBase) )
		return FALSE;

	// update directory flag
	BOOL bUpdateDirectory = FALSE;

	// get current directory
	if ( bVerbose )
		INFOSTR( "downloading directory\n" );
	if ( !cRio.RxDirectory() )
	{
		ERRORSTR( "RxDirectory() failed\n" );
		return FALSE;
	}

	// if init request
	if ( bInit )
	{
		if ( bVerbose )
			INFOSTR( "initializing\n" );

		cRio.Initialize();

		bUpdateDirectory = TRUE;
	}

	// if upload file request
	if ( pszFileUpload )
	{
		if ( bVerbose )
			INFOSTR( "uploading %s\n", pszFileUpload );

		if ( !cRio.TxFile(pszFileUpload, bVerbose) )
			return FALSE;

		bUpdateDirectory = TRUE;
	}

	// if dowwnload file request
	if ( pszTrackNot )
	{
		if ( bVerbose )
			INFOSTR( "downloading file\n");

		if ( !cRio.RxFile(pszTrackNo, bVerbose) )
			return FALSE;
	}


	// if playlist request
	if ( pszFilePlaylist )
	{
		if ( !ProcessPlaylist(cRio, pszFilePlaylist, bVerbose) )
			return FALSE;

		bUpdateDirectory = TRUE;
	}

	// if directory update required
	if ( bUpdateDirectory )
	{
		if ( bVerbose )
			INFOSTR( "updating directory\n" );

		if ( !cRio.TxDirectory() )
		{
			ERRORSTR( "TxDirectory() failed\n" );
			return FALSE;
		}
	}

	// if display directory request
	if ( bDisplayDir )
		DisplayDirectory( cRio, bVerbose );

	// restore
	cRio.Unset();

	return TRUE;
}

